package com.getperka.sea.ext;

/*
 * #%L
 * Simple Event Architecture - JMS Support
 * %%
 * Copyright (C) 2012 Perka Inc.
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;

import com.getperka.sea.Event;
import com.getperka.sea.impl.SerializableEventTransport;
import com.google.inject.ImplementedBy;

/**
 * Defines how {@link Event} instances can be written and read from streams. The default
 * implementation of EventTransport support types that implements {@link Serializable}.
 */
@ImplementedBy(SerializableEventTransport.class)
public interface EventTransport {
  /**
   * Return {@code true} if the given event can can be transported by the implementation.
   */
  boolean canTransport(Class<? extends Event> eventType);

  /**
   * Construct an {@link Event} instance from a stream.
   */
  Event decode(InputStream input) throws EventTransportException;

  /**
   * Write an {@link Event} into an {@link OutputStream}.
   * 
   * @return {@code false} if the event should not be sent
   */
  boolean encode(Event event, EventContext context, OutputStream output)
      throws EventTransportException;

  /**
   * Return a string uniquely identifying the type. The value returned from this method need not be
   * reversible.
   */
  String getTypeName(Class<? extends Event> eventType);
}
